import "core-js/modules/es6.regexp.replace";
import "core-js/modules/es6.string.link";
import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _possibleConstructorReturn from "@babel/runtime/helpers/possibleConstructorReturn";
import _getPrototypeOf from "@babel/runtime/helpers/getPrototypeOf";
import _inherits from "@babel/runtime/helpers/inherits";
import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _objectSpread from "@babel/runtime/helpers/objectSpread";
import "core-js/modules/es7.symbol.async-iterator";
import "core-js/modules/es6.symbol";
import "core-js/modules/web.dom.iterable";

var _dec, _dec2, _dec3, _class, _class2, _temp;

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
import { types } from "mobx-state-tree";
import React, { Component, Fragment } from 'react';
import { observer, inject } from "mobx-react";
import PropTypes from 'prop-types';
import Drawer from '@material-ui/core/Drawer';
import ListItemText from '@material-ui/core/ListItemText';
import MenuList from '@material-ui/core/MenuList';
import MenuItem from '@material-ui/core/MenuItem';
import ChevronLeft from '@material-ui/icons/ChevronLeft';
import ChevronRight from '@material-ui/icons/ChevronRight';
import ListItemIcon from '@material-ui/core/ListItemIcon';
import withStyles from '@material-ui/core/styles/withStyles';
import ExpandLess from '@material-ui/icons/ExpandLess';
import ExpandMore from '@material-ui/icons/ExpandMore';
import Collapse from '@material-ui/core/Collapse';
import Link from './Link';
import classnames from 'classnames';
import AmpMenu from './amp/AmpMenu';
import withTheme from '@material-ui/core/styles/withTheme';
export var MenuItemModel = types.model("MenuItemModel", {
  text: types.optional(types.string, ''),
  url: types.maybeNull(types.string),
  state: types.frozen(),
  image: types.maybeNull(types.string),
  items: types.maybeNull(types.array(types.late(function () {
    return MenuItemModel;
  }))),
  root: types.optional(types.boolean, false),
  server: types.optional(types.boolean, false),
  prefetch: types.maybeNull(types.string),
  expanded: false
}).actions(function (self) {
  return {
    toggle: function toggle() {
      self.expanded = !self.expanded;
    },
    collapse: function collapse() {
      self.expanded = false;
    }
  };
});
export var MenuModel = types.model("MenuModel", {
  open: false,
  levels: types.optional(types.array(MenuItemModel), []),
  level: types.optional(types.number, 0)
}).actions(function (self) {
  return {
    /**
     * Closes the menu
     */
    close: function close() {
      self.open = false;
    },

    /**
     * Toggles the open state of the menu
     */
    toggle: function toggle() {
      self.open = !self.open;
    },

    /**
     * Updates the root node
     * @param {Object} root 
     */
    setRoot: function setRoot(root) {
      self.levels[0] = MenuItemModel.create(root);
    },

    /**
     * Selects an item in the menu
     * @param {MenuItem} item
     * @param {Object} options
     */
    setSelected: function setSelected(item) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      item = MenuItemModel.create(item.toJSON());
      self.level++;

      if (self.levels.length <= self.level) {
        self.levels.push(item);
      } else {
        self.levels[self.level] = item;
      }

      if (options.expandFirstItem && item.items.every(function (itm) {
        return itm.expanded === false;
      })) {
        item.items[0].expanded = true;
      }
    },

    /**
     * Goes back one level
     */
    goBack: function goBack() {
      self.level = Math.max(0, self.level - 1);
      self.collapseAll();
    },

    /**
     * Closes all expandable sections
     */
    collapseAll: function collapseAll() {
      setTimeout(function () {
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = self.levels[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var level = _step.value;
            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
              for (var _iterator2 = level.items[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                var item = _step2.value;
                item.collapse();
              }
            } catch (err) {
              _didIteratorError2 = true;
              _iteratorError2 = err;
            } finally {
              try {
                if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
                  _iterator2.return();
                }
              } finally {
                if (_didIteratorError2) {
                  throw _iteratorError2;
                }
              }
            }
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return != null) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }
      }, 200);
    }
  };
});
export var styles = function styles(theme) {
  return {
    drawer: {
      zIndex: theme.zIndex.modal + 20,
      display: 'flex',
      flexDirection: 'column',
      borderTop: "".concat(theme.headerHeight, "px solid transparent"),
      'body.moov-safari &': {
        // Turning off momentum scrolling on iOS here to fix frozen body issue
        // Source: https://moovweb.atlassian.net/browse/PRPL-342
        '-webkit-overflow-scrolling': 'auto'
      }
    },
    list: {
      flex: 'none',
      overflowY: 'auto',
      overflowX: 'hidden',
      padding: 0
    },
    listPadding: {
      padding: 0
    },
    header: {
      position: 'absolute',
      left: '10px',
      top: '12px'
    },
    icon: {
      marginRight: '0'
    },
    headerText: {
      textAlign: 'center',
      fontWeight: '600',
      textTransform: 'uppercase',
      fontSize: theme.typography.body1.fontSize
    },
    hbox: {
      display: 'flex',
      flexDirection: 'row',
      transition: 'all ease-out .2s'
    },
    listItem: {
      textTransform: 'uppercase',
      lineHeight: 'initial',
      fontSize: theme.typography.body1.fontSize
    },
    link: {
      textDecoration: 'none',
      color: 'inherit'
    },
    listItemImage: {
      width: '40px',
      height: '40px',
      marginRight: 0
    },
    listItemIcon: {
      marginRight: 0
    },
    expander: {
      backgroundColor: "".concat(theme.palette.primary.paper, " !important")
    },
    leaf: _objectSpread({
      textTransform: 'none'
    }, theme.typography.body1),
    expanded: {
      backgroundColor: "".concat(theme.palette.secondary.main, " !important"),
      color: theme.palette.secondary.contrastText,
      '& svg': {
        color: theme.palette.secondary.contrastText
      }
    },
    drawerFixed: {
      top: 0,
      height: '100vh',
      borderTop: 'none'
    }
  };
};
/**
 * The main app menu that slides in from the left when the AppHeader's menu button is clicked.
 */

var Menu = (_dec = withTheme(), _dec2 = withStyles(styles, {
  name: 'RSFMenu'
}), _dec3 = inject('app'), _dec(_class = _dec2(_class = _dec3(_class = observer(_class = (_temp = _class2 =
/*#__PURE__*/
function (_Component) {
  _inherits(Menu, _Component);

  function Menu() {
    var _getPrototypeOf2;

    var _this;

    _classCallCheck(this, Menu);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _possibleConstructorReturn(this, (_getPrototypeOf2 = _getPrototypeOf(Menu)).call.apply(_getPrototypeOf2, [this].concat(args)));

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "slideToItem", function (item, menu) {
      var expandFirstItem = _this.props.expandFirstItem;
      menu.setSelected(item, {
        expandFirstItem: expandFirstItem
      });
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "goBack", function () {
      _this.props.app.menu.goBack();
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "toggleItemExpaned", function (item) {
      item.toggle();
    });

    return _this;
  }

  _createClass(Menu, [{
    key: "render",
    value: function render() {
      var _this2 = this;

      var _this$props = this.props,
          app = _this$props.app,
          classes = _this$props.classes,
          align = _this$props.align,
          rootHeader = _this$props.rootHeader,
          rootFooter = _this$props.rootFooter,
          drawerWidth = _this$props.drawerWidth,
          simple = _this$props.simple,
          persistent = _this$props.persistent;
      var amp = app.amp,
          menu = app.menu;
      var levels = menu.levels,
          level = menu.level;
      var position = -drawerWidth * level;

      if (!menu) {
        return null;
      } else if (amp) {
        return React.createElement(AmpMenu, this.props);
      }

      return React.createElement(Drawer, {
        variant: persistent ? 'persistent' : 'temporary',
        open: menu.open || persistent,
        onClose: menu.close,
        anchor: align,
        ModalProps: {
          keepMounted: true
        },
        PaperProps: {
          style: {
            width: "".concat(drawerWidth, "px")
          }
        },
        classes: {
          paper: classnames(classes.drawer, _defineProperty({}, classes.drawerFixed, persistent))
        }
      }, simple ? this.renderSimple() : React.createElement("div", {
        className: classes.hbox,
        style: {
          transform: "translateX(".concat(position, "px)"),
          flex: 1
        }
      }, levels.map(function (list, depth) {
        return React.createElement(MenuList, {
          style: {
            width: "".concat(drawerWidth, "px")
          },
          classes: {
            root: classes.list,
            padding: classes.padding
          },
          key: depth
        }, list.root && rootHeader, !list.root && React.createElement(MenuItem, {
          divider: true,
          button: true,
          onClick: _this2.goBack
        }, React.createElement(ListItemIcon, {
          classes: {
            root: classes.header
          }
        }, React.createElement(ChevronLeft, {
          className: classes.icon
        })), React.createElement(ListItemText, {
          classes: {
            root: classes.headerText
          },
          primary: React.createElement("div", {
            className: classes.headerText
          }, list.text, " ")
        })), list.items && list.items.map(function (item, key) {
          return _this2.renderItem(depth, item, key);
        }), list.root && rootFooter);
      })), this.renderLinksForSEO());
    }
    /**
     * @private
     * Renders simple anchor tags for all links in a collapsed div so that
     * search engine bots can crawl them.
     */

  }, {
    key: "renderLinksForSEO",
    value: function renderLinksForSEO() {
      var levels = this.props.app.menu.levels;
      var root = levels.length && levels[0];
      if (!root) return null;
      var links = [],
          key = 0;

      var findLinks = function findLinks(_ref) {
        var items = _ref.items;

        for (var i = 0; i < items.length; i++) {
          var item = items[i];

          if (item.url) {
            links.push(React.createElement("li", {
              key: key++
            }, React.createElement(Link, {
              to: item.url
            }, item.text)));
          }

          if (item.items) {
            findLinks(item);
          }
        }
      };

      findLinks(root);
      return React.createElement("noscript", null, React.createElement("ul", null, links));
    }
    /**
     * Renders the menu as a simple list of expandable sections
     * @return {MenuList}
     */

  }, {
    key: "renderSimple",
    value: function renderSimple() {
      var _this3 = this;

      var _this$props2 = this.props,
          rootHeader = _this$props2.rootHeader,
          rootFooter = _this$props2.rootFooter,
          menu = _this$props2.app.menu,
          classes = _this$props2.classes;
      var root = menu && menu.levels && menu.levels[0];
      if (!root) return null;
      return React.createElement(MenuList, {
        classes: {
          padding: classes.list
        }
      }, rootHeader, root.items.map(function (item, i) {
        return _this3.renderItem(1, item, i);
      }), rootFooter);
    }
    /**
     * Renders a menu item
     * @param {Numbere} depth The depth of the item in the menu
     * @param {MenuItem} item The menu item
     * @param {String} key a key for the react element
     * @return {React.Element}
     */

  }, {
    key: "renderItem",
    value: function renderItem(depth, item, key) {
      if (item.items) {
        return this.renderGroup(depth, item, key);
      }

      return this.renderLeaf(item, key);
    }
  }, {
    key: "renderGroup",
    value: function renderGroup(depth, item, key) {
      var _classnames2;

      var _this$props3 = this.props,
          menu = _this$props3.app.menu,
          classes = _this$props3.classes,
          useExpanders = _this$props3.useExpanders,
          simple = _this$props3.simple;
      var showExpander = simple || depth > 0 && useExpanders;
      var elements = [React.createElement(MenuItem, {
        key: key,
        button: true,
        divider: true,
        onClick: showExpander ? this.toggleItemExpaned.bind(this, item) : this.slideToItem.bind(this, item, menu),
        classes: {
          root: classnames(classes.listItem, (_classnames2 = {}, _defineProperty(_classnames2, classes.expanded, item.expanded), _defineProperty(_classnames2, classes.expander, showExpander), _classnames2))
        }
      }, this.renderItemContents(item, false, showExpander))];

      if (showExpander) {
        elements.push(React.createElement(Collapse, {
          in: item.expanded,
          timeout: "auto",
          key: "".concat(key, "-collapse")
        }, React.createElement(MenuList, {
          component: "div",
          classes: {
            root: classes.list
          }
        }, item.items && item.items.map(this.renderItem.bind(this, depth + 1)))));
      }

      return elements;
    }
  }, {
    key: "renderLeaf",
    value: function renderLeaf(item, key) {
      var _this$props4 = this.props,
          classes = _this$props4.classes,
          location = _this$props4.app.location;
      return React.createElement(Link, {
        key: key,
        to: item.url,
        className: classes.link,
        server: item.server,
        state: item.state ? function () {
          return JSON.parse(item.state);
        } : null
      }, React.createElement(MenuItem, {
        button: true,
        divider: true,
        selected: location.pathname === item.url.replace(/\?.*/, ''),
        classes: {
          root: classnames(classes.listItem, classes.leaf)
        }
      }, this.renderItemContents(item, true)));
    }
  }, {
    key: "renderItemContents",
    value: function renderItemContents(item, leaf, showExpander) {
      var _this$props5 = this.props,
          itemRenderer = _this$props5.itemRenderer,
          classes = _this$props5.classes,
          ExpandIcon = _this$props5.ExpandIcon,
          CollapseIcon = _this$props5.CollapseIcon,
          theme = _this$props5.theme;
      var contents;

      if (itemRenderer) {
        contents = itemRenderer(item, leaf);
      }

      if (contents) {
        return contents;
      } else if (leaf) {
        return React.createElement(Fragment, null, item.image && React.createElement(ListItemIcon, null, React.createElement("img", {
          className: classes.listItemImage,
          alt: item.text,
          src: item.image
        })), React.createElement(ListItemText, {
          primary: item.text,
          disableTypography: true
        }));
      } else {
        ExpandIcon = ExpandIcon || theme.ExpandIcon || ExpandMore;
        CollapseIcon = CollapseIcon || theme.CollapseIcon || ExpandLess;
        return React.createElement(Fragment, null, item.image && React.createElement(ListItemIcon, null, React.createElement("img", {
          className: classes.listItemImage,
          alt: item.text,
          src: item.image
        })), React.createElement(ListItemText, {
          primary: item.text,
          disableTypography: true
        }), React.createElement(ListItemIcon, {
          className: classes.listItemIcon
        }, showExpander ? item.expanded ? React.createElement(CollapseIcon, {
          className: classes.icon
        }) : React.createElement(ExpandIcon, {
          className: classes.icon
        }) : React.createElement(ChevronRight, {
          className: classes.icon
        })));
      }
    }
  }]);

  return Menu;
}(Component), _defineProperty(_class2, "propTypes", {
  menu: PropTypes.object,

  /**
   * The width of the drawer in pixels
   */
  drawerWidth: PropTypes.number,

  /**
   * An element to display at the top of the root of the menu
   */
  rootHeader: PropTypes.element,

  /**
   * An element to display at the bottom of the root of the menu
   */
  rootFooter: PropTypes.element,

  /**
   * Set to true to use expandable menu items below depth 1
   */
  useExpanders: PropTypes.bool,

  /**
   * Set to true to expand first item for not root items
   */
  expandFirstItem: PropTypes.bool,

  /**
   * Set to true to display the menu
   */
  open: PropTypes.bool,

  /**
   * Set to true to dock the menu so that it's always open and not modal
   */
  persistent: PropTypes.bool,

  /**
   * CSS classes for this component
   */
  classes: PropTypes.objectOf(PropTypes.string),

  /**
   * Called when the menu is closed
   */
  onClose: PropTypes.func,

  /**
   * Set to true to render a simple set of expanders rather than a multi-level drill down.
   * Defaults to false.
   */
  simple: PropTypes.bool,

  /**
   * The icon to use for collapsed groups
   */
  ExpandIcon: PropTypes.func,

  /**
   * The icon to use for expanded groups
   */
  CollapseIcon: PropTypes.func,

  /**
   * Sets the side of the screen from which the menu appears.
   */
  align: PropTypes.oneOf(['left', 'right']),

  /**
   * A function to render the contents of a menu item.  It is passed the following arguments:
   * 
   * 1.) item - the MenuItemModel instance. 
   * 2.) leaf - `true` when the item is a leaf node, otherwise `false`
   * 
   * Return undefined to render the default contents
   * 
   * Example:
   * 
   *  itemRenderer={(item, leaf) => {
   *    return leaf ? <ListItemText primary={item.text}/> : undefined
   *  }}
   */
  itemRenderer: PropTypes.func
}), _defineProperty(_class2, "defaultProps", {
  drawerWidth: 330,
  simple: false,
  expandFirstItem: false,
  align: 'left'
}), _temp)) || _class) || _class) || _class) || _class);
export { Menu as default };
//# sourceMappingURL=Menu.js.map