import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _possibleConstructorReturn from "@babel/runtime/helpers/possibleConstructorReturn";
import _getPrototypeOf from "@babel/runtime/helpers/getPrototypeOf";
import _inherits from "@babel/runtime/helpers/inherits";
import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import _defineProperty from "@babel/runtime/helpers/defineProperty";

var _dec, _dec2, _class, _temp;

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
import React, { Component, Fragment } from 'react';
import { inject, observer, Provider } from 'mobx-react';
import { Helmet } from 'react-helmet';
import withStyles from '@material-ui/core/styles/withStyles';
import CssBaseline from '@material-ui/core/CssBaseline';
import { canUseClientSideNavigation } from './utils/url';
import delegate from 'delegate';
import { cache } from './router/serviceWorker';
import { isSafari } from './utils/browser';
/**
 * @private
 * Internal PWA root used when launching the app.  Do not use this class directly
 */

export var styles = function styles(theme) {
  return {
    '@global': {
      'body.moov-modal': {
        overflow: 'hidden',
        position: 'fixed',
        maxWidth: '100vw',
        maxHeight: '100vh'
      },
      'body.moov-blur #root': {
        filter: 'blur(5px)',
        transition: "filter ".concat(theme.transitions.duration.enteringScreen, "ms")
      }
    }
  };
};
var PWA = (_dec = withStyles(styles), _dec2 = inject(function (_ref) {
  var app = _ref.app,
      history = _ref.history;
  return {
    menu: app.menu,
    app: app,
    history: history,
    amp: app.amp
  };
}), _dec(_class = _dec2(_class = observer(_class = (_temp =
/*#__PURE__*/
function (_Component) {
  _inherits(PWA, _Component);

  function PWA() {
    var _getPrototypeOf2;

    var _this;

    _classCallCheck(this, PWA);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _possibleConstructorReturn(this, (_getPrototypeOf2 = _getPrototypeOf(PWA)).call.apply(_getPrototypeOf2, [this].concat(args)));

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "_nextId", 0);

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "nextId", function () {
      return _this._nextId++;
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "onRouteChange", function (_location, action) {
      if (action === 'PUSH') {
        window.scrollTo(0, 0);

        _this.props.menu.close();
      }
    });

    return _this;
  }

  _createClass(PWA, [{
    key: "render",
    value: function render() {
      var _this$props = this.props,
          amp = _this$props.amp,
          app = _this$props.app;
      return React.createElement(Provider, {
        nextId: this.nextId
      }, React.createElement(Fragment, null, React.createElement(CssBaseline, null), React.createElement(Helmet, null, React.createElement("html", {
        lang: "en"
      }), React.createElement("meta", {
        charset: "utf-8"
      }), React.createElement("meta", {
        name: "viewport",
        content: "width=device-width,initial-scale=1,minimum-scale=1,shrink-to-fit=no"
      }), React.createElement("meta", {
        name: "theme-color",
        content: "#000000"
      }), app.description ? React.createElement("meta", {
        name: "description",
        content: app.description
      }) : null, app.canonicalURL ? React.createElement("link", {
        rel: "canonical",
        href: app.canonicalURL
      }) : null, React.createElement("link", {
        rel: "manifest",
        href: "/manifest.json"
      }), React.createElement("title", null, app.title)), amp && React.createElement(Helmet, null, React.createElement("script", {
        async: true,
        src: "https://cdn.ampproject.org/v0.js"
      }), React.createElement("script", {
        async: true,
        "custom-element": "amp-install-serviceworker",
        src: "https://cdn.ampproject.org/v0/amp-install-serviceworker-0.1.js"
      })), amp && React.createElement("amp-install-serviceworker", {
        src: "https://".concat(app.location.hostname, "/service-worker.js"),
        "data-iframe-src": "https://".concat(app.location.hostname, "/pwa/install-service-worker.html"),
        layout: "nodisplay"
      }), this.props.children));
    }
  }, {
    key: "componentDidCatch",
    value: function componentDidCatch(error, info) {
      var app = this.props.app;
      app.applyState({
        page: 'Error',
        error: error.message,
        stack: info.componentStack
      });
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      // Send state to service worker to cache
      this.props.history.listen(this.onRouteChange);
      this.watchLinkClicks(); // put os class on body for platform-specific styling

      this.addDeviceClassesToBody(); // cache the launch screen for when the pwa is installed on the desktop

      cache('/?source=pwa');
    }
    /**
     * Adds a css class corresponding to the browser to the body element
     * @private
     */

  }, {
    key: "addDeviceClassesToBody",
    value: function addDeviceClassesToBody() {
      if (isSafari()) {
        document.body.classList.add('moov-safari');
      }
    }
    /**
     * Returns true if client-side navigation should be forced, otherwise false
     * @param {HTMLElement} linkEl
     * @return {Boolean} 
     */

  }, {
    key: "shouldNavigateOnClient",
    value: function shouldNavigateOnClient(linkEl) {
      var href = linkEl.getAttribute('href');
      var linkTarget = linkEl.getAttribute('target'); // false if the element is not a link

      if (linkEl.tagName.toLowerCase() !== 'a') return false; // false if the link was rendered by react-storefront/Link - it will handle nav on its own

      if (linkEl.getAttribute('data-moov-link') === 'on') return false; // false if link has data-reload="on|true"

      if (['true', 'on'].indexOf(linkEl.getAttribute('data-reload')) !== -1) return false; // false for links with a target other than self

      if (linkTarget && linkTarget !== '_self') return false;
      return canUseClientSideNavigation(href);
    }
    /**
     * Watches for clicks on all links and forces client-side navigation if the domain is the same.
     * This behavior can be overridden by adding data-reload="on" to any link
     */

  }, {
    key: "watchLinkClicks",
    value: function watchLinkClicks() {
      var _this2 = this;

      // capture click events
      delegate('a', 'click', function (e) {
        var delegateTarget = e.delegateTarget;

        if (_this2.shouldNavigateOnClient(delegateTarget)) {
          // don't reload the page
          e.preventDefault(); // instead do the navigation client-side using the history API

          _this2.props.history.push(delegateTarget.getAttribute('href'));
        }
      });
    }
  }]);

  return PWA;
}(Component), _temp)) || _class) || _class) || _class);
export { PWA as default };
//# sourceMappingURL=PWA.js.map