import "core-js/modules/es6.regexp.replace";
import "core-js/modules/es6.regexp.search";

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
import React from 'react';
import { hydrate } from './renderers';
import { connectReduxDevtools } from "mst-middlewares";
import createBrowserHistory from 'history/createBrowserHistory';
import registerServiceWorker from './registerServiceWorker';
import PWA from './PWA';
import { onSnapshot } from 'mobx-state-tree';
import debounce from 'lodash/debounce';
/**
 * Bootstraps the PWA react application.
 * @param {options} options
 * @param {React.Element} options.app The root app element
 * @param {Object} options.theme A material-ui theme
 * @param {Object} options.model A mobx-state-tree model class
 * @param {HTMLElement} options.target The DOM element to mount onto
 */

export default function launchClient(_ref) {
  var App = _ref.App,
      theme = _ref.theme,
      model = _ref.model,
      router = _ref.router,
      _ref$target = _ref.target,
      target = _ref$target === void 0 ? document.getElementById('root') : _ref$target;
  var history = createBrowserHistory();
  var state = hydrate({
    component: React.createElement(PWA, null, React.createElement(App, null)),
    model: model,
    theme: theme,
    target: target,
    providerProps: {
      history: history,
      router: router
    }
  });

  if (process.env.NODE_ENV !== 'production') {
    connectReduxDevtools(require("remotedev"), state);
    window.addEventListener('unhandledrejection', function (event) {
      return window.moov.state.onError(event.reason);
    });
  } // record app state in history.state restore it when going back or forward
  // see Router#onLocationChange


  onSnapshot(state, debounce(function (snapshot) {
    var _history$location = history.location,
        pathname = _history$location.pathname,
        search = _history$location.search;

    if (!snapshot.loading) {
      history.replace(pathname + search, snapshot);
    }
  }, 150));
  window.addEventListener('load', function () {
    // we only start watching after the window.onload event so that
    // timing metrics are fully collected and be reported correctly to analytics
    router.watch(history, state.applyState);
  });
  registerServiceWorker();
}
//# sourceMappingURL=launchClient.js.map