import "core-js/modules/es6.object.assign";
import _objectSpread from "@babel/runtime/helpers/objectSpread";
import "core-js/modules/es6.regexp.search";

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
import { types, onPatch } from "mobx-state-tree";
import SelectionModelBase from "./SelectionModelBase";
var ProductModelBase = types.model('ProductModelBase', {
  id: types.identifier,
  url: types.maybeNull(types.string),
  name: types.maybeNull(types.string),
  description: types.maybeNull(types.string),
  size: types.maybeNull(SelectionModelBase),
  color: types.maybeNull(SelectionModelBase),
  quantity: types.optional(types.number, 1),
  rating: types.maybeNull(types.number),
  reviewCount: types.maybeNull(types.number),
  basePrice: types.maybeNull(types.number),
  brand: types.maybeNull(types.string),
  images: types.array(types.string),
  thumbnails: types.array(types.string),
  thumbnail: types.maybeNull(types.string),
  loadingImages: false
}).views(function (self) {
  return {
    shouldApplyPatchOnPop: function shouldApplyPatchOnPop(patch) {
      return patch.page === 'Product';
    },

    get price() {
      var size = self.size && self.size.selected;

      if (size && size.price) {
        return size.price;
      } else {
        return self.basePrice;
      }
    }

  };
}).actions(function (self) {
  return {
    afterCreate: function afterCreate() {
      onPatch(self, function (patch) {
        if (patch.path === '/color/selected') {
          // fetch images when a new color is selected
          self.fetchImages();
        }
      });
    },

    /**
     * Updates the quantity and fires an analytics event
     * @param {Number} quantity 
     */
    setQuantity: function setQuantity(q) {
      self.quantity = q < 1 ? 1 : q;
    },

    /**
     * Get images for the selected color
     */
    fetchImages: function fetchImages() {
      var _window$location = window.location,
          pathname = _window$location.pathname,
          search = _window$location.search;
      var selected = self.color.selected;
      self.loadingImages = true;

      if (selected) {
        fetch("".concat(pathname, "/images/").concat(selected.id, ".json").concat(search)).then(function (res) {
          return res.json();
        }).then(function (state) {
          return self.apply(_objectSpread({}, state, {
            loadingImages: false
          }));
        }).catch(function () {
          return self.apply({
            loadingImages: false
          });
        });
      }
    },

    /**
     * Update the images based on the response from the images handler
     * @param {String[]} images 
     */
    apply: function apply(state) {
      Object.assign(self, state);
    }
  };
});
export default ProductModelBase;
//# sourceMappingURL=ProductModelBase.js.map