import _regeneratorRuntime from "@babel/runtime/regenerator";
import "regenerator-runtime/runtime";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
// The default callback used when none is provided.  Simply returns the upstream HTML unaltered.
var perfectProxy = function perfectProxy(_params, _request, response) {
  return response.send();
};
/**
 * A handler that fetches HTML from the same path as the current request on the upstream site. Use
 * this handler to transform HTML from the upstream site or return it unaltered.
 *   
 * The callback you provide is passed the following arguments:
 * 
 * params {Object} Request parameters parsed from the route and query string
 * request {Object} An object representing the request
 * response {Response} An object representing the response.  Call response.send(html) to send the resulting html back to the browser.
 * 
 * To use response from the upstream site unaltered, called response.send() with no arguments.
 * 
 * @param {Function} cb A callback which takes params, request, and response.
 */


export default function proxyUpstream() {
  var cb = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : perfectProxy;
  return {
    type: 'proxyUpstream',
    runOn: {
      server: true,
      client: true
    },
    fn: function () {
      var _fn = _asyncToGenerator(
      /*#__PURE__*/
      _regeneratorRuntime.mark(function _callee(params, request, response) {
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (!(process.env.MOOV_RUNTIME === 'client')) {
                  _context.next = 4;
                  break;
                }

                // reload the page if this handler is called during client-side navigation
                window.moov.router.reloadFromServer();
                _context.next = 14;
                break;

              case 4:
                if (!(cb == null)) {
                  _context.next = 6;
                  break;
                }

                throw new Error('You must provide a path to a handler in proxyUpstream().  Please check your routes.');

              case 6:
                // indicate handler path and asset class in a response header so we can track it in logs
                response.set('x-rsf-handler', cb.path);
                response.set('x-rsf-response-type', 'proxy');
                _context.next = 10;
                return cb(params, request, response);

              case 10:
                _context.t0 = _context.sent;

                if (_context.t0) {
                  _context.next = 13;
                  break;
                }

                _context.t0 = {
                  proxyUpstream: true
                };

              case 13:
                return _context.abrupt("return", _context.t0);

              case 14:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function fn(_x, _x2, _x3) {
        return _fn.apply(this, arguments);
      }

      return fn;
    }()
  };
}
//# sourceMappingURL=proxyUpstream.js.map